#
# shinGETsu - P2P anonymous BBS
#
# (c) FukTommy <fuktommy@users.sourceforge.net>
# Released under the GNU General Public License
# $Id: Signature.pm,v 1.3 2004/02/17 12:19:12 fuktommy Exp $
#

package Shingetsu::Signature;
use IPC::Open2;
use Shingetsu::Config;
use Shingetsu::Extern;

$trustlist = $Shingetsu::Config::trust;
$apollo	   = $Shingetsu::Config::command{apollo};

#
# make sigunature
#
sub generate($$) {
	my($keygen, $str) = @_;
	open2($IN, $OUT, $apollo, "-g");
	print $OUT "$keygen\n";
	close $OUT;
	my $pubkey = <$IN>;
	my $prikey = <$IN>;
	close $IN;
	wait;
	chomp($pubkey, $prikey);

	my $md5 = Shingetsu::Extern::md5digest($str);
	open2($IN, $OUT, $apollo, "-s");
	print $OUT "$md5\n$pubkey\n$prikey\n";
	close $OUT;
	my $sign = <$IN>;
	close $IN;
	wait;
	chomp($sign);
	return ($pubkey, $sign);
}

#
# check sigunature
#
sub check(%) {
	my(%rec) = @_;
	my($str, $t) = ("", "");
	foreach $t (split /,/, $rec{target}) {
		$str .= "<>$t:$rec{$t}";
	}
	$str =~ s/^<>//;
	my $md5 = Shingetsu::Extern::md5digest($str);
	open2($IN, $OUT, $apollo, "-v");
	print $OUT "$md5\n$rec{sign}\n$rec{pubkey}\n";
	close $OUT;
	my $flag = <$IN>;
	close $IN;
	wait;
	chomp($flag);
	return ($flag eq "True");
}

#
# make public key readable string
#
sub pubkey2trip($) {
	my($pubkey) = @_;
	if (defined &Shingetsu::Extern::pubkey2trip) {
		return Shingetsu::Extern::pubkey2trip($pubkey)
	} else {
		open2($IN, $OUT, $apollo, "-c");
		print $OUT "$pubkey\n";
		close $OUT;
		my $trip = <$IN>;
		close $IN;
		wait;
		chomp($trip);
		return $trip;
	}
}

#
# add trip to trust list
#
sub addlist($) {
	my($trip) = @_;
	local $_;
	unless (include($trip)) {
		Shingetsu::Util::lock();
		open OUT, ">> $trustlist" or die "$trustlist: open failed. $!";
		print OUT "$trip\n";
		close OUT;
		Shingetsu::Util::unlock();
	}
}

#
# remove trip from list
#
sub dellist($) {
	my($trip) = @_;
	$trip .= "\n";
	local $_;
	my @trip;
	if (-e $trustlist) {
		open IN, $trustlist or exit;
		@trip = <IN>;
		close IN;
	} else {
		return;
	}
	if (grep $_ eq $trip, @trip) {
		Shingetsu::Util::lock();
		open OUT, "> $trustlist" or die "$trustlist: open failed. $!";
		foreach (@trip) {
			print OUT if ($_ ne $trip);
		}
		close OUT;
		Shingetsu::Util::unlock();
	}
}

#
# include trip
#
sub include($) {
	my($trip) = @_;
	my @trip = list();
	if (grep $_ eq $trip, @trip) {
		return 1;
	} else {
		return undef;
	}
}

#
# show all trip
#
sub list() {
	local $_;
	open IN, $trustlist or return ();
	my @trip = <IN>;
	chomp @trip;
	return @trip;
}

1;
